import 'package:built_collection/built_collection.dart';

extension ListBuilderExtensions<T> on ListBuilder<T> {
  void replaceItem(T item, T replacement) {
    assert(item != null);
    assert(replacement != null);
    map((element) => element == item ? replacement : element);
  }
}

extension BuiltListExtensions<T> on BuiltList<T> {
  BuiltList<T> replaceItem(T item, T replacement) {
    assert(replacement != null);
    assert(item != null);
    return (toBuilder()..replaceItem(item, replacement)).build();
  }

  BuiltList<T> append(T item) {
    assert(!contains(item));
    return (toBuilder()..add(item)).build();
  }

  BuiltList<T> without(T item) {
    assert(contains(item));
    return (toBuilder()..remove(item)).build();
  }
}
