import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';

import '../serializers.dart';
import 'loading_status.dart';

part 'loading_state.g.dart';

abstract class LoadingState
    implements Built<LoadingState, LoadingStateBuilder> {
  static Serializer<LoadingState> get serializer => _$loadingStateSerializer;

  factory LoadingState.error(String errorMessage, [String errorDetails]) =>
      LoadingState._builder((b) => b
        ..errorMessage = errorMessage
        ..errorDetails = errorDetails
        ..status = LoadingStatus.error);

  factory LoadingState.initial() =>
      LoadingState._builder((b) => b.status = LoadingStatus.initial);

  factory LoadingState.loading() =>
      LoadingState._builder((b) => b.status = LoadingStatus.loading);
  factory LoadingState.ready() =>
      LoadingState._builder((b) => b.status = LoadingStatus.ready);

  LoadingState._();

  factory LoadingState._builder([void Function(LoadingStateBuilder) updates]) =>
      _$LoadingState(updates);

  @nullable
  String get errorDetails;

  @nullable
  String get errorMessage;

  LoadingStatus get status;

  Map<String, dynamic> toJson() {
    return blocExtraSerializers.serializeWith(LoadingState.serializer, this);
  }

  static LoadingState fromJson(Map<String, dynamic> json) {
    return blocExtraSerializers.deserializeWith(LoadingState.serializer, json);
  }
}
