import 'package:built_collection/built_collection.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';

import '../common/common.dart';
import '../serializers.dart';

part 'simple_list_state.g.dart';

abstract class SimpleListState<T>
    implements Built<SimpleListState<T>, SimpleListStateBuilder<T>> {
  BuiltList<T> get items;
  LoadingState get loadingState;

  SimpleListState._();
  factory SimpleListState.builder([
    void Function(SimpleListStateBuilder<T>) updates,
  ]) = _$SimpleListState<T>;

  factory SimpleListState.initial() => SimpleListState.builder(
        (b) => b.loadingState.replace(LoadingState.initial()),
      );

  factory SimpleListState.loading([
    Iterable<T> items,
  ]) =>
      SimpleListState.builder((b) => b
        ..items.replace(items ?? const [])
        ..loadingState.replace(LoadingState.loading()));

  factory SimpleListState.ready([
    Iterable<T> items,
  ]) =>
      SimpleListState.builder((stateBuilder) => stateBuilder
        ..items.replace(items)
        ..loadingState.replace(LoadingState.ready()));

  factory SimpleListState.error({
    Iterable<T> items,
    String error,
    String details,
  }) =>
      SimpleListState.builder((stateBuilder) => stateBuilder
        ..items.replace(items)
        ..loadingState.replace(LoadingState.error(error, details)));

  Map<String, dynamic> toJson() {
    return blocExtraSerializers.serializeWith(SimpleListState.serializer, this);
  }

  static SimpleListState fromJson(Map<String, dynamic> json) {
    return blocExtraSerializers.deserializeWith(
        SimpleListState.serializer, json);
  }

  static Serializer<SimpleListState> get serializer =>
      _$simpleListStateSerializer;
}
