import 'package:bloc_extras/bloc_extras.dart';
import 'package:built_value/built_value.dart';
import 'package:built_value/serializer.dart';
import 'package:built_value/standard_json_plugin.dart';
import 'package:test/test.dart';

part 'nesting_serializers_test.g.dart';

void main() {
  group('Bloc extra serializers', () {
    final state = SimpleListState.error(
      details: 'An error detail',
      error: 'Serialized error',
      items: [
        TestItem(1, 'Item 1'),
      ],
    );

    tearDown(() {
      blocExtraSerializers = null;
    });

    test('Showld throw a serialization error', () {
      expect(() => state.toJson(), throwsStateError);
    });

    test('Showld serialize normally', () {
      blocExtraSerializers = serializers;
      expect(() => state.toJson(), isNot(throws));
    });
  });
}

abstract class TestItem implements Built<TestItem, TestItemBuilder> {
  int get id;
  String get name;

  TestItem._();
  factory TestItem([
    int id,
    String name,
  ]) =>
      _$TestItem((b) => b
        ..id = id
        ..name = name);

  Map<String, dynamic> toJson() {
    return serializers.serializeWith(TestItem.serializer, this);
  }

  static TestItem fromJson(Map<String, dynamic> json) {
    return serializers.deserializeWith(TestItem.serializer, json);
  }

  static Serializer<TestItem> get serializer => _$testItemSerializer;
}

@SerializersFor([
  TestItem,
])
final Serializers serializers = (_$serializers.toBuilder()
      ..addPlugin(StandardJsonPlugin())
      ..applyBlocExtraSerializers())
    .build();
